local LibStub = LibStub
local AceAddon = LibStub("AceAddon-3.0")
local AceLocale = LibStub("AceLocale-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local ControlPanel = AceAddon:GetAddon("ControlPanel")
local ControlPanel_Options = AceAddon:GetAddon("ControlPanel_Options")
local L = AceLocale:GetLocale("ControlPanel_Options")
local get_cvar_bool = ControlPanel_Options.get_cvar_bool
local string_format = string.format
local GetClassInfoByID = GetClassInfoByID
local sound_channels = {["24"]=L["Low (24)"],["32"]=L["Medium (32)"],["64"]=L["High (64)"]}
local Sound_GameSystem_GetNumOutputDrivers = Sound_GameSystem_GetNumOutputDrivers

local function get_output_drivers_table()
	local t = Sound_GameSystem_GetNumOutputDrivers()
	local r = {}
	local i
	for i=1,t do
		local s = Sound_GameSystem_GetOutputDriverNameByIndex(i-1)
		r[s] = s
	end
	return r
end

local order = 0
local function get_order()
	local temp = order
	order = order +1
	return temp
end

local Sound =
{
	order = get_order(),
	name = L["Sound"],
	desc = L["ControlPanel only manages Volumes and Class Voices."],
	type = "group",
	args =
	{
		Sound_EnableAllSound =
		{
			order = get_order(),
			name = L["Enable"],
			desc = L["Enables or disables all sound."],
			type = "toggle",
			set = function(info,val)
				SetCVar("Sound_EnableAllSound",val)
			end,
			get = function(info)
				return get_cvar_bool("Sound_EnableAllSound")
			end,
			width = "full",
		},
		Sound_MasterVolume =
		{
			order = get_order(),
			name = L["Master Volume"],
			type = "range",
			min = 0,
			max = 1,
			isPercent = true,
			set = function(info,val)
				ControlPanel:SetCVar("Sound_MasterVolume",val)
			end,
			get = function(info)
				return ControlPanel:GetCVarNumber("Sound_MasterVolume")
			end,
			width = "full",
		},
		SoundEffects =
		{
			order = get_order(),
			name = L["Sound Effects"],
			type = "group",
			args = 
			{
				Sound_EnableSFX = 
				{
					order = get_order(),
					name = L["Enable"],
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnableSFX",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnableSFX")
					end,
					width = "full",
				},
				Sound_SFXVolume =
				{
					order = get_order(),
					name = L["Volume"],
					type = "range",
					min = 0,
					max = 1,
					isPercent = true,
					set = function(info,val)
						ControlPanel:SetCVar("Sound_SFXVolume",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("Sound_SFXVolume")
					end,
					width = "full",
				},
				Sound_EnablePetSounds =
				{
					order = get_order(),
					name = L["Pet"],
					desc = L["Toggles pet idle and aggro sounds on and off."],
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnablePetSounds",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnablePetSounds")
					end,
				},
				Sound_EnableEmoteSounds =
				{
					order = get_order(),
					name = L["Emote"],
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnableEmoteSounds",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnableEmoteSounds")
					end,
				},
			}
		},
		Music =
		{
			order = get_order(),
			name = L["Music"],
			type = "group",
			args = 
			{
				Sound_EnableMusic =
				{
					order = get_order(),
					name = L["Enable"],
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnableMusic",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnableMusic")
					end,
					width = "full",
				},
				Sound_MusicVolume =
				{
					order = get_order(),
					name = L["Volume"],
					type = "range",
					min = 0,
					max = 1,
					isPercent = true,
					set = function(info,val)
						ControlPanel:SetCVar("Sound_MusicVolume",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("Sound_MusicVolume")
					end,
					width = "full",
				},
				Sound_ZoneMusicNoDelay = 
				{
					order = get_order(),
					name = L["Loop"],
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_ZoneMusicNoDelay",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_ZoneMusicNoDelay")
					end,
				},
				Sound_EnablePetBattleMusic = 
				{
					order = get_order(),
					name = L["Pet Battle"],
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnablePetBattleMusic",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnablePetBattleMusic")
					end,
				},
			}
		},
		Ambience = 
		{
			order = get_order(),
			name = L["Ambience"],
			type = "group",
			args = 
			{
				Sound_EnableAmbience =
				{
					order = get_order(),
					name = L["Enable"] ,
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnableAmbience",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnableAmbience")
					end,
					width = "full",
				},
				Sound_AmbienceVolume =
				{
					order = get_order(),
					name = L["Volume"],
					type = "range",
					min = 0,
					max = 1,
					isPercent = true,
					set = function(info,val)
						ControlPanel:SetCVar("Sound_AmbienceVolume",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("Sound_AmbienceVolume")
					end,
					width = "full",
				},
			}
		},
		Dialog =
		{
			order = get_order(),
			name = L["Dialog"],
			type = "group",
			args = 
			{
				Sound_EnableDialog =
				{
					order = get_order(),
					name = L["Enable"] ,
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnableDialog",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnableDialog")
					end,
					width = "full",
				},
				Sound_DialogVolume =
				{
					order = get_order(),
					name = L["Volume"],
					type = "range",
					min = 0,
					max = 1,
					isPercent = true,
					set = function(info,val)
						ControlPanel:SetCVar("Sound_DialogVolume",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("Sound_DialogVolume")
					end,
					width = "full",
				},
				Sound_EnableErrorSpeech = 
				{
					order = get_order(),
					name = L["Error Speech"] ,
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnableErrorSpeech",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnableErrorSpeech")
					end,
				},
			}
		},
		Playback =
		{
			order = get_order(),
			name = L["Playback"],
			desc = L["ControlPanel won't manage these options."],
			type = "group",
			args = 
			{
				Sound_EnableSoundWhenGameIsInBG = 
				{
					order = get_order(),
					name = L["Sound in Background"],
					desc = L["Enable to allow sounds to play even when World of Warcraft is in the background."],
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnableSoundWhenGameIsInBG",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnableSoundWhenGameIsInBG")
					end,
					width = "full",
				},
				Sound_EnableReverb = 
				{
					order = get_order(),
					name = L["Reverb"],
					desc = L["This will affect performance."],
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnableReverb",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnableReverb")
					end,
					width = "full",
				},
				Sound_EnableSoftwareHRTF = 
				{
					order = get_order(),
					name = L["Headphone Mode"],
					desc = L["Enables software emulation for headphone surround sound."],
					type = "toggle",
					set = function(info,val)
						SetCVar("Sound_EnableSoftwareHRTF",val)
					end,
					get = function(info)
						return get_cvar_bool("Sound_EnableSoftwareHRTF")
					end,
					width = "full",
				},
			}
		},
		ClassVoices = 
		{
			order = get_order(),
			name = L["Class Voices"],
			desc = L["Special voice modulation effects for some classes. Disable to improve performance."],
			type = "group",
			args =
			{
				Sound_EnableDSPEffects = 
				{
					order = get_order(),
					name = GetClassInfoByID(6),
					type = "toggle",
					set = function(info,val)
						ControlPanel:SetCVar("Sound_EnableDSPEffects",val)
					end,	
					get = function(info)
						return ControlPanel:GetCVarBool("Sound_EnableDSPEffects")
					end,
				}
			}
		},
		Hardware =
		{
			order = get_order(),
			name = L["Hardware"],
			desc = L["Requires a restart of the game's sound systems"] ,
			type = "group",
			args =
			{
				Sound_OutputDriverName =
				{
					order = get_order(),
					name = L["Game Sound Output"],
					type = "select",
					values = get_output_drivers_table,
					set = function(info,val)
						SetCVar("Sound_OutputDriverName",val)
					end,
					get = function(info)
						return GetCVar("Sound_OutputDriverName")
					end,
					width = "full",
				},
				Sound_NumChannels = 
				{
					order = get_order(),
					name = L["Sound Channels"],
					type = "select",
					values = sound_channels,
					set = function(info,val)
						SetCVar("Sound_NumChannels",val)
					end,
					get = function(info)
						return GetCVar("Sound_NumChannels")
					end,
					width = "full",
				},
				restart_sound_systems =
				{
					order = get_order(),
					name = L["Restart sound systems"],
					type = "execute",
					func = function(info)
						Sound_GameSystem_RestartSoundSystem()
					end,
					width = "full"
				}
			}
		}
	}
}


ControlPanel_Options:push("Sound",Sound)